/*
 *
 * This file is part of the Percentile Noise Estimation algorithm.
 *
 * Copyright(c) 2012 Miguel Colom.
 * miguel.colom@cmla.ens-cachan.fr
 *
 * This file may be licensed under the terms of of the
 * GNU General Public License Version 2 (the ``GPL'').
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the GPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the GPL along with this
 * program. If not, go to http://www.gnu.org/licenses/gpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 */

#include <fftw3.h>
#include <stdlib.h>
#include "CFFT.h"

CFFT::CFFT(int Nx, int Ny) {
  this->Nx = Nx;
  this->Ny = Ny;

  // Get memory for FFT buffers
  this->FFT_in = (fftwf_complex *)fftwf_malloc(sizeof(fftwf_complex) * Nx*Ny);
  if (this->FFT_in == NULL) {
    printf("CFFT unable to fftwf_malloc memory for FFT_in (%d bytes)\n",
            sizeof(fftwf_complex) * Nx*Ny);
    exit(-1);
  }  
  this->FFT_out = (fftwf_complex *)fftwf_malloc(sizeof(fftwf_complex) * Nx*Ny);
  if (this->FFT_out == NULL) {
    printf("CFFT unable to fftwf_malloc memory for FFT_out (%d bytes)\n",
            sizeof(fftwf_complex) * Nx*Ny);
    exit(-1);
  }  
  this->FFT_tmp = (fftwf_complex *)fftwf_malloc(sizeof(fftwf_complex) * Nx*Ny);
  // this->FFT_tmp can be NULL. It will must be checked outside.

  // Forward and backward plans
  this->p_forward = fftwf_plan_dft_2d(Ny, Nx,
                                      FFT_in, FFT_out,
                                      FFTW_FORWARD, FFTW_ESTIMATE);  
  this->p_backward = fftwf_plan_dft_2d(Ny, Nx,
                                      FFT_in, FFT_out,
                                      FFTW_BACKWARD, FFTW_ESTIMATE);
}

fftwf_complex *CFFT::get_buffer_in() {
  return this->FFT_in; 
}

fftwf_complex *CFFT::get_buffer_out() {
  return this->FFT_out;
}

fftwf_complex *CFFT::get_buffer_tmp() {
  return this->FFT_tmp;
}

void CFFT::FFT_forward() {
  fftwf_execute(this->p_forward);
}

void CFFT::FFT_backward() {
  fftwf_execute(this->p_backward);  
}

CFFT::~CFFT() {
  fftwf_destroy_plan(this->p_backward);
  fftwf_destroy_plan(this->p_forward);
  fftwf_free(this->FFT_out);
  fftwf_free(this->FFT_in);
  if (this->FFT_tmp != NULL) fftwf_free(this->FFT_tmp);
}

