/*
 *
 * This file is part of the Percentile Noise Estimation algorithm.
 *
 * Copyright(c) 2011 Miguel Colom.
 * miguel.colom@cmla.ens-cachan.fr
 *
 * This file may be licensed under the terms of of the
 * GNU General Public License Version 2 (the ``GPL'').
 *
 * Software distributed under the License is distributed
 * on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
 * express or implied. See the GPL for the specific language
 * governing rights and limitations.
 *
 * You should have received a copy of the GPL along with this
 * program. If not, go to http://www.gnu.org/licenses/gpl.html
 * or write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 */

#ifndef FILTERS_H
#define	FILTERS_H

#include "CFFT.h"
#include "framework/CFramework.h"

//! Computes the discrete divergence of image igray into
//! igray2 using finite differences.
/*!
  \param *igray Original input image
  \param *igray2 Filtered output image
  \param width Width of the image
  \param height Height of the image
*/

void noise_estimate_compute_derivative(float *igray, float *igray2,
                                           int width, int height);

//! Computes the Laplacian of image igray into igray2 using finite differences.
/*!
  \param *igray Original input image
  \param *igray2 Filtered output image
  \param width Width of the image
  \param height Height of the image
*/
void noise_estimate_compute_laplacian(float *igray, float *igray2,
                                          int width, int height);
//! Filter test stencil.
/*!
  \param *igray Original input image
  \param *igray2 Filtered output image
  \param width Width of the image
  \param height Height of the image
*/
float *fill_filter_new_test(int *support);

//! Computes the Laplacian of Laplacian of image igray into igray2
//! using finite differences.
/*!
  \param *igray Original input image
  \param *igray2 Filtered output image
  \param width Width of the image
  \param height Height of the image
*/
void noise_estimate_compute_laplacian2(float *igray, float *igray2,
                                           int width, int height);

//! Computes the Laplacian of Laplacian of Laplacian of image igray into
//! igray2 using finite differences.
/*!
  \param *igray Original input image
  \param *igray2 Filtered output image
  \param width Width of the image
  \param height Height of the image
*/
void noise_estimate_compute_laplacian3(float *igray, float *igray2,
                                           int width, int height);

//! Computes the Laplacian of Laplacian of Laplacian of Laplacina of image
//! igray into igray2 using finite differences.
/*!
  \param *igray Original input image
  \param *igray2 Filtered output image
  \param width Width of the image
  \param height Height of the image
*/
void noise_estimate_compute_laplacian4(float *igray, float *igray2,
                                           int width, int height);

float *get_filter(int *L, int filter_type);

//! Prefilters the image using the specified filter
/*!
  \param *u Original image
  \param *out Output filtered image
  \param Nx Width of the image
  \param Ny Height of the image
  \param *filter Filter operator
  \param L Filter support
  \param *FFT Helper FFT object
*/
void prefilter(float *u, float *out,
                int Nx, int Ny,
                float *filter, int L, CFFT *FFT);

/*
void save_debug_FFT(fftwf_complex *img, int COLS, int ROWS,
                    const char *filename, float mult);
void save_debug_FFT(float *img, int COLS, int ROWS,
                    const char *filename, float mult);
*/

#endif	/* FILTERS_H */

